      SUBROUTINE C_PHYSICS(ABORT,err)
*--------------------------------------------------------
*-
*-   Purpose and Methods : Compute coincident quantities
*-
*-   Output: ABORT           - success or failure
*-         : err             - reason for failure, if any
*- 
* $Log: c_physics.f,v $
* Revision 1.10  2003/12/19 19:21:04  jones
* Use ssinplane and hsinplane in calculating cqx,cqz and cmissing_momx,cmissing_momz
*
* Revision 1.9  2003/11/28 15:49:41  jones
* Go back to adding h_oopcentral_offset and s_oopcentral_offset to hsxp_tar and
* ssxp_tar since this in no longer done in h_physics.f and s_physics.f (MKJ)
*
* Revision 1.8  2003/09/05 21:54:44  jones
* Remove phi_offset addition to xptar. It is done in h_physics.f and s_physics.f (mkj)
*
* Revision 1.7  1996/09/04 15:31:12  saw
* (JRA) Add phi_offset to calculation of q 3 vector
*
* Revision 1.6  1996/04/29 19:13:00  saw
* (JRA) Corrections
*
* Revision 1.5  1996/01/22 15:08:02  saw
* (JRA) Adjust variable names.  Get particle properties from lookup
* tables
*
* Revision 1.4  1996/01/16 21:06:55  cdaq
* (RE?) Change some definitions
*
* Revision 1.3  1995/05/22 20:50:44  cdaq
* (SAW) Split gen_data_data_structures into gen, hms, sos, and coin parts"
*
* Revision 1.2  1995/05/13  02:44:07  cdaq
* (SAW) Remove some debugging
*
* Revision 1.1  1995/05/11  15:17:36  cdaq
* Initial revision
*
*--------------------------------------------------------
      IMPLICIT NONE
      SAVE
*
      character*9 here
      parameter (here= 'C_PHYSICS')
*
      logical ABORT
      character*(*) err
*
      logical deuterium
      logical flag1
      logical flag2
*
      include 'gen_data_structures.cmn'
      include 'hms_data_structures.cmn'
      include 'sos_data_structures.cmn'
      include 'coin_data_structures.cmn'
      include 'gen_constants.par'
      include 'hms_scin_tof.cmn'
      include 'sos_scin_tof.cmn'
      include 'hms_physics_sing.cmn'
      include 'sos_physics_sing.cmn'
      include 'hms_scin_parms.cmn'
      include 'sos_scin_parms.cmn'
*
*     local variables
*
      real*4 cqx,cqy,cqz,cqabs
      real*4 ekinrec,m_rec
      real*4 tar_amin1
      real*4 offset_ctime

      real*4 mp,mp2,mn,mn2,md,md2,mpi0,mpi02,meta,meta2,mpi,mpi2
      real*4 deg_rad,amu,alpha,c_pi
      real*4 dummy,dot
      real*4 missingmass2,mmx2
      real*4 targmass           ! this target mass is the mass of 
*                               ! the nucleon the pion recoils against
      real*4 new_x_x,new_x_y,new_x_z,new_y_x,new_y_y,new_y_z
      real*4 new_z_x,new_z_y,new_z_z,p_new_x,p_new_y,p_new_z
      real*4 p_e,p_h,energy_e,energy_h,theta_e,theta_h
      real*4 phi_e,phi_h,xp_e_tar,xp_h_tar,m_hadron,sign_hadron
      real*4 cqxzabs,phx,phy,phz
      real*4 cmissingmom
      real*4 cmissingmomoop
      real*4 cgammav

      parameter (mp   = 0.93827231)           ! all masses in GeV/c^2
      parameter (mp2  = mp*mp)
      parameter (mn   = 0.93956563)
      parameter (mn2  = mn*mn)
      parameter (md   = 1.87561339)
      parameter (md2  = md*md)
      parameter (mpi0 = 0.1349764)
      parameter (mpi02  = mpi0*mpi0)
      parameter (meta = 0.54745)
      parameter (meta2  = meta*meta)
      parameter (deg_rad = 3.1415927/180.)
      parameter (mpi = 0.13956995)
      parameter (mpi2 = mpi*mpi)
      parameter (amu = 0.93149432)
      parameter (alpha = 0.00072973531)
      parameter (c_pi = 3.1415926536)

**************************************************

      ABORT = .FALSE.
      err = ' '

      deuterium = .false.
      flag1 = .false.
      flag2 = .false.

      if(HSNUM_FPTRACK.le.0.or.SSNUM_FPTRACK.le.0) then
        return
      endif

*     Need to select which arm is the hadron

*      tar_amin1= gtarg_a(gtarg_num)-1.0
*      m_rec = tar_amin1*m_amu
      
      targmass    = 0.          ! target nucleon mass
      sign_hadron = 0.          ! pi+/pi-/p
      m_rec       = 0.          ! recoil mass
      c_e_bind    = 0.          ! binding energy
      cmex        = 0.
      cmmx        = 0.
      ce_excx     = 0.
      cmissingmom = 0.
      cmissingmomoop = 0.
      cgammav     = 0.
     
      if ((gtarg_num.eq.13).or.
     &     (gtarg_num.eq.15).or.
     &     (gtarg_num.eq.16)) then

! TH - uncomment this if running dummy target for ld2
!      if ((gtarg_num.eq.18).or.(gtarg_num.eq.13).or.(gtarg_num.eq.15)
!     >      .or.(gtarg_num.eq.16)) then

         c_e_bind=0.00222445
         deuterium = .true.

      endif
      
      if ((hpartmass.lt.2*mass_electron) .and.
     &     (spartmass.gt.2*mass_electron)) then ! HMS electron arm
         p_e         = hsp
         p_h         = ssp
         energy_e    = hsenergy
         energy_h    = ssenergy

*         write(*,*)'uncorr hsxp_tar=',hsxp_tar
         xp_e_tar    = hsxp_tar + hphicentral_offset ! done by JV
         xp_h_tar    = ssxp_tar + sphicentral_offset ! done by JV
*         write(*,*)'corr hsxp_tar=',xp_e_tar
*         write(*,*)'++++++++++++++++++++++++++++++++'
*         xp_e_tar    = hsxp_tar + h_oopcentral_offset ! done by TH
*         xp_h_tar    = ssxp_tar + s_oopcentral_offset ! done by TH

*         theta_e     = hstheta ! done by JV
*         theta_h     = -sstheta ! done by JV

         theta_e     = hsinplane ! done by TH
         theta_h     = -ssinplane ! done by TH

         phi_e       = hsphi
         phi_h       = ssphi
         m_hadron    = spartmass
         sign_hadron = sbfield 

      else if ((spartmass.lt.2*mass_electron) .and.
     &        (hpartmass.gt.2*mass_electron)) then ! SOS electron arm
         p_e         = ssp
         p_h         = hsp
         energy_e    = ssenergy
         energy_h    = hsenergy
*         write(*,*)'uncorr ssxp_tar=',ssxp_tar
         xp_e_tar    = ssxp_tar + sphicentral_offset ! done by JV
         xp_h_tar    = hsxp_tar + hphicentral_offset ! done by JV
*         write(*,*)'corr ssxp_tar=',xp_e_tar
*         write(*,*)'++++++++++++++++++++++++++++++++'

*         xp_e_tar    = ssxp_tar + s_oopcentral_offset ! done by TH 
*         xp_h_tar    = hsxp_tar + h_oopcentral_offset ! done by TH

*         theta_e     = -sstheta ! done by JV
*         theta_h     = hstheta ! done by JV

         theta_e     = -ssinplane ! done by TH 
         theta_h     = hsinplane ! done by TH 

         phi_e       = ssphi
         phi_h       = hsphi
         m_hadron    = hpartmass
         sign_hadron = hbfield

      else
         write(6,*) 'c_physics: no electron arm'
         return
      endif

c        if(m_hadron.lt.0.8) then
c        write(6,*) "hadron mass=",m_hadron       
c        endif


* CHECK FOR PION PRODUCTION OR ELASTIC SCATTERING. IF PION PRODUCTION, SET
* RECOIL PARTICLE MASS TO NEUTRON AS DEFAULT, OPTIONAL CHANGING PION CHARGE
* SIGN AND RECOIL MASS BY SUPPLYING THE HADRON ARM MAGNETIC FIELD SIGN IN
* THE DBASE/*.kinematics FILE.

* targmass is actually the mass of the target nucleus in (quasi)free 
* scattering, therefore p for pi+, n for pi-

*      write(6,*) "ssp=",ssp
      if(abs(m_hadron-mpi).lt.0.01) then
         m_hadron = mpi
         if (sign_hadron.ge.0.) then
            targmass=mp
            m_rec=mn
            flag1 = .true.
c     else if ((sign_hadron.lt.0.).and.deuterium) then
         else if (sign_hadron.le.0.) then
            targmass=mn
            m_rec=mp
            flag2 = .true.
         else
*     write(6,*) 'c_physics: return at prod/elast'
            return
         endif
      endif    
      
      if (abs(m_hadron-mp) .lt. 0.01) then
         m_hadron = mp
         targmass = mp
         m_rec=mp
      endif
      
c       if(m_hadron.lt.1.8) then
c       write(6,*) "hadron mass=",m_hadron  
c       endif



      cqx = -p_e*cos(xp_e_tar)*sin(theta_e)
      cqy = -p_e*sin(xp_e_tar)
      cqz = gpbeam - p_e*cos(xp_e_tar)*cos(theta_e)
      cqabs= sqrt(cqx**2+cqy**2+cqz**2)
      
      phx =  p_h*cos(xp_h_tar)*sin(theta_h)
      phy =  p_h*sin(xp_h_tar)
      phz =  p_h*cos(xp_h_tar)*cos(theta_h)
      
      cmissing_momx = cqx - phx
      cmissing_momy = cqy - phy
      cmissing_momz = cqz - phz
      
      cmissingmom    = sqrt(cmissing_momx**2 + cmissing_momy**2 
     >     + cmissing_momz**2)

      if (abs(cmissingmom).lt.10000.) then
         cmissing_mom=cmissingmom
      else
      endif

      cmissing_mom_par = (cmissing_momx*cqx+cmissing_momz*cqz)/cqabs
      cmissing_mom_perp = (-cmissing_momz*cqx+cmissing_momx*cqz)/cqabs

      cmissing_mom1=sqrt(cmissing_mom_par**2.+cmissing_mom_perp**2.)
      
c   we want pm_perp > 0 when pm closer to downstream beam than q
c   effectively the sign introduced by the if statement is qx/abs(qx)
c   in this way pm_perp equals pm_x', where the primed coordinate system
c   which is used to calculate phi_pi will be defined later, in case q
c   lies in the x-z plane (qy=0)
      
      cmissingmomoop  =   cmissing_momy
      
      if (abs(cmissingmomoop).lt.10000.) then
         cmissing_mom_oop=cmissingmomoop
      else
      endif
      
      if(abs(gebeam-energy_e).lt.1000.) then
         c_omega = gebeam-energy_e
      else
      endif

      cmissing_e= c_omega + targmass - sqrt(p_h**2+m_hadron**2) !miss'n energy


!     added by CB on 04/17/2007
****************************************
      if (abs(cqabs*cqabs - c_omega*c_omega).lt.10000.) then
         c_bigq2 = cqabs*cqabs - c_omega*c_omega
      else
      endif
*      write(*,*)'Q2 =',c_bigq2
 
      c_w2 = targmass**2 + 2*targmass*c_omega - c_bigq2 

      if (c_w2.gt.0.) then 
         c_invmass = sqrt(c_w2)                             !W = inv mass
      elseif (c_w2.eq.0.) then
         c_invmass = 0. 
      else
         c_invmass = 1e-6 
*         write(6,*)'c_physics: Got negative W2!'
      endif

      x_bj = c_bigq2/(2.0*mp*c_omega)
*      write(*,*)'X_bj =',x_bj
      qabs = sqrt(c_bigq2+c_omega*c_omega) ! exchanged virtual photon momentum
      z_m = hsenergy/c_omega 
      pt2 = cmissing_mom_perp*cmissing_mom_perp ! perp comp of missing momentum squared

      if ((gtarg_num.eq.17).or.(gtarg_num.eq.18)) then
         write(6,*) 'TH: engine.f - Dummy target!!'
        if(flag1) then
           targmass = mp
        elseif(flag2) then
           targmass = mn
        endif
      endif
      
      if(deuterium) cmex=c_omega+md-sqrt(p_h**2+m_hadron**2)
         
****************************************



!     added by CB on 04/17/2007
*****************************************
*     COMPUTING THE VIRTUAL PHOTON POLARIZATION - EPSILON

      if((1.+2.*cqabs**2/c_bigq2*(tan(theta_e/2.))**2).eq.0.) write(6,*) 
     >     'c_physics: epsilon problem'
      
      c_epsilon=1./(1.+2.*cqabs**2/c_bigq2*(tan(theta_e/2.))**2)
      c_epsilon=min(c_epsilon,0.99)
      
*     COMPUTING THE VIRTUAL PHOTON FLUX FACTOR - GAMNA_V
      
      cgammav=alpha/2./c_pi**2*(c_w2-targmass**2)/c_bigq2*energy_e/gebeam
     >     /(1.-c_epsilon)

      if (abs(cgammav).lt.10000.) then
      c_gamma_v=cgammav
      else
      endif
c     e_pion_lab = sqrt(p_h**2 + mpi2)                    ! Pion Energy
      
*     Computing the THETA & PHI between the scattering plane and reaction plane
c     therefore define a new system with the z axis parallel to q, and
c     the x axis inside the q-z-plane
c     z' = q, y' = q X z, x' = y' X q
c     this gives phi the way it is usually defined, i.e. phi=0 for in-plane
c     particles closer to the downstream beamline than q
c     phi=90 is above the horizontal plane when q points to the right, and
c     below the horizontal plane when q points to the left
      

************************ THETA_PQ ***************************

      dot = phx*cqx + phy*cqy + phz*cqz

      if((cqabs*p_h).eq.0.) write(6,*) 'c_physics: dot problem'

      if(abs(dot/(cqabs*p_h)).le.1.0) then
        cthetapq = acos(dot/(cqabs*p_h))
      else
        cthetapq = -1.
      endif
c      write(6,*) 'c_physics: at 5'

************************ PHI_PQ ***************************

      dummy=sqrt( (cqx**2+cqy**2) * cqabs**2 )
      if(dummy.eq.0.) write(6,*) 'c_physics: dummy x = 0'
      new_x_x = -cqx*cqz/dummy
      new_x_y = -cqy*cqz/dummy
      new_x_z = (cqx**2 + cqy**2)/dummy

      dummy   = sqrt(cqx**2 + cqy**2)
      if(dummy.eq.0.) write(6,*) 'c_physics: dummy y = 0'
      new_y_x =  cqy/dummy
      new_y_y = -cqx/dummy
      new_y_z =  0.

      if(cqabs.eq.0.) write(6,*) 'c_physics: dummy z = 0'
      new_z_x = cqx/cqabs
      new_z_y = cqy/cqabs
      new_z_z = cqz/cqabs

      p_new_x = phx*new_x_x + phy*new_x_y + phz*new_x_z
      p_new_y = phx*new_y_x + phy*new_y_y + phz*new_y_z
      p_new_z = phx*new_z_x + phy*new_z_y + phz*new_z_z

      if ((p_new_x**2+p_new_y**2).eq.0.) then
         write(6,*) 'c_physics: dummy cphipi = 0'
         cphipq = 0.
      else
         cphipq = acos(p_new_x/sqrt(p_new_x**2+p_new_y**2))
      endif
      if (p_new_y.lt.0.) then
         cphipq = 2*c_pi - cphipq
      endif
c      write(6,*) 'c_physics: at 6'

!     added by CB on 04/17/2007
****************************************

      missingmass2=cmissing_e**2-cmissing_mom**2 
      
      if (m_hadron.eq.mpi) then
         if (missingmass2.le.0) then
            cmissing_mass = -.1
         else
            cmissing_mass = sqrt(missingmass2)
         endif
         
         if (deuterium) then
            mmx2 = cmex**2 - cmissing_mom**2
            if (mmx2.gt.0.) then
               cmmx = sqrt(mmx2)
            else
               cmmx = -.2
            endif
         endif
         
         ce_exc = cmissing_mass-m_rec-c_e_bind
         if (deuterium) ce_excx = cmmx-2.*m_rec
         
      elseif ((m_hadron.eq.mp).and.(missingmass2.ne.0.)) then
         
         cmissing_mass=abs(missingmass2)/missingmass2*sqrt(abs(missingmass2))
         ce_exc = cmissing_mass
         
      else
         cmissing_mass = 0.
         ce_exc = 0.
         
      endif

*****************************************

! ----------------------------------------------------------------------------
! Missing Mass for A>1
!         mmx2=(cmissing_e)**2 - cmissing_mom**2 !!! Nucleon missing mass
!         if (mmx2.gt.0.) then
!           cmmx = sqrt(mmx2)
!           print *,'w = ', c_omega ,' mt = ' , targmass, ' Phadron = ', p_h
!         print *,'Mhadron = ', m_hadron ,' Em = ' , cmissing_e, 
!     >          ' Pm = ', cmissing_mom
!           print *,'MA-1 = ', m_rec, ' mmx = ', cmmx
!        else
!           cmmx = -20000.
!        endif    
! ----------------------------------------------------------------------------




!     added by CB on 04/17/2007
****************************************

*     Computing Mandelstam variable t = (P_n-P_p)^2 represents the four- momentum
*     transfer to the nucleonic system
        
c        cmin_t = c_bigq2 - m_hadron**2 + 
c     >           2*c_omega*sqrt(p_h**2+m_hadron**2) - 2*dot

      cmin_t=-(mp-mn)**2+2*targmass*(sqrt(m_rec**2+cmissing_mom**2)-m_rec)

c        t = (c_bigq2 - hsp*cos(theta_gp_lab))**2 +
c     >      (hsp*sin(theta_gp_lab))**2 - (omega-e_pion_lab)**2 

      ctphix= - cmin_t * cos(cphipq)
      ctphiy=   cmin_t * sin(cphipq)
      
c     write(6,*)'c_phys: at 8'
      
      
***********************************************************
***********************************************************
 
*     Coincidence timing.
      
*      offset_ctime = - (hstime_at_fp-hstart_time_center) ! new CVS version
*     &     + (sstime_at_fp-sstart_time_center)
*     &     - hspath_cor + sspath_cor + 10
      
      offset_ctime = - (hstime_at_fp-hstart_time_center)
     &     + (sstime_at_fp-sstart_time_center)
     &     - hspath_cor + sspath_cor + 10. - 22.7 ! old Volmer's replay_4 version
      
*      ccointime_hms = (hmisc_dec_data(10,1)-2450)/9.46 + offset_ctime ! new CVS version
      ccointime_hms = (hmisc_dec_data(10,1)-1960)/9.46 + offset_ctime 
     &     + 0.14881 ! old Volmer's replay_4 version   
*      write(*,*)'before corr cointime=',ccointime_hms   
*      write(*,*)'test_cointime =',ccointime_hms_shift
      ccointime_hms = (hmisc_dec_data(10,1)-1960)/9.46 + offset_ctime 
     &     - ccointime_hms_shift + 0.14881 ! old Volmer's replay_4 version

*      write(*,*)'after corr  cointime=',ccointime_hms  
*      write(*,*)'++++++++++++++++++++++++++'      
*      ccointime_sos = (smisc_dec_data(9,1)-1570)/9.68 - offset_ctime ! new CVS version
      
      ccointime_sos = (smisc_dec_data(9,1)-2062.5)/9.68 - offset_ctime 
     &     - ccointime_sos_shift ! old Volmer's replay_4 version
      
      
      return
      end
      
