      SUBROUTINE S_SATCORR(ABORT,err)
*--------------------------------------------------------
*-
*-   Purpose and Methods : Correct delta or other reconstructed physics variables
*-                         for magnet saturation effects
*-                              
*-                                to decoded information 
*-
*-      Required Input BANKS     SOS_FOCAL_PLANE
*-                               SOS_TARGET
*-
*-      Output BANKS             SOS_PHYSICS_R4
*-                               SOS_PHYSICS_I4
*-
*-   Output: ABORT           - success or failure
*-         : err             - reason for failure, if any
*- 
*-   Created 24-JUN-1998   J. Volmer
*-                           Dummy Shell routine
* $Log: s_satcorr.f,v $
* Revision 1.2  2003/09/05 20:00:03  jones
* Merge in online03 changes (mkj)
*
* Revision 1.1.2.1  2003/04/11 14:01:34  cdaq
* eliminate p0corr since already in s_fieldcorr.f (mkj)
*
* Revision 1.1  1999/02/10 18:35:01  csa
* Initial revision
*
*
* Revision 1.1  1998/06/24  14:58:18  jv
* Initial revision
*
*-
*-
*--------------------------------------------------------
      IMPLICIT NONE
      SAVE
*
      character*9 here
      parameter (here= 'S_SATCORR')
*
      REAL*4 tmp1,tmp2,tmp3,tmp4
      logical ABORT
      character*(*) err
      integer ierr
*
      include 'gen_data_structures.cmn'
      INCLUDE 'sos_data_structures.cmn'
      INCLUDE 'gen_constants.par'
      INCLUDE 'gen_units.par'
*
*     local variables 
*
      REAL*4 deltacorr,p0corr

*--------------------------------------------------------
*
      ierr=0
      tmp1=0.0
      tmp2=0.0
      tmp3=0.0
      tmp4=0.0

      ABORT=.FALSE.

      p0corr=0.
      deltacorr=0.

      if (genable_sos_satcorr.eq.1999) then
         
*         write(*,*)'Using Volmer 1999 correction'
         
         if (spcentral.gt.0.96296) then 
            deltacorr = 46.729-137.21*spcentral+181.15*spcentral**2
     >           -76.089*spcentral**3
         else
            deltacorr = 14.6369
         endif
         
         
      elseif (genable_sos_satcorr.eq.2003) then
         
         write(*,*)'Using Tanja 2003 correction'
         
         if (spcentral.gt.0.96296) then
            deltacorr = 46.729-137.21*spcentral+181.15*spcentral**2
     >           -76.089*spcentral**3
         else
            deltacorr = 14.6369
         endif

         ssdelta = ssdelta + deltacorr*ssxp_fp**2  
         
      elseif (genable_sos_satcorr.eq.2005) then
         
         write(*,*)'Using Ben  2005 correction'
         
         tmp1=0.0199-0.34515*ssxp_tar+12.82*ssxp_tar**2.  
     >        +212.492*ssxp_tar**3.
         tmp2=(-0.01602-0.27781*ssxp_tar -14.7455*ssxp_tar**2.)
     >        *spcentral
         tmp3=-174.44*exp(138.856*(ssxp_tar-0.0755))+0.1113
     >        +6.1344*ssxp_tar
         tmp4=(1.077-1.561*spcentral+0.3876*spcentral**3.)
     >        *exp(0.4756*spcentral)
         
         deltacorr=tmp1+tmp2+tmp3*tmp4
         
*         write(*,*)'deltacorr Clasie=',deltacorr
         
         ssdelta = ssdelta*(1+deltacorr/100) 
         
      endif
  

      ABORT= ierr.ne.0 .or. ABORT
      
      return
      end
